import django_filters

from django.conf import settings
from publieditor.content.models import Brand, CarModel, Car
from publieditor.content.settings import (
    PUBLIEDITOR_CAR_SHAPE_CHOICES,
    PUBLIEDITOR_CAR_TARGET_CHOICES,
    PUBLIEDITOR_CAR_SEAT_CHOICES,
    PUBLIEDITOR_CAR_ENGINE_CHOICES,
    PUBLIEDITOR_CAR_TRACTION_CHOICES,
    PUBLIEDITOR_CAR_AGEAR_CHOICES,
    PUBLIEDITOR_CAR_ENGINE_LOC_CHOICES,
    PUBLIEDITOR_CAR_DOORS_CHOICES,
    PUBLIEDITOR_CAR_FUEL_CHOICES,
    PUBLIEDITOR_CAR_POLLUTION_CHOICES,
    PUBLIEDITOR_CAR_LAYOUT_CHOICES,
)

class CarFilter(django_filters.FilterSet):
    BRAND_CHOICES = Brand.get_brand_choices_list()
    brand = django_filters.MultipleChoiceFilter(name="brand", choices=BRAND_CHOICES)
    model = django_filters.NumberFilter(name="model")
    min_price = django_filters.NumberFilter(name="price", lookup_type='gte')
    max_price = django_filters.NumberFilter(name="price", lookup_type='lte')
    shape = django_filters.MultipleChoiceFilter(name="shape", choices=PUBLIEDITOR_CAR_SHAPE_CHOICES)
    target = django_filters.MultipleChoiceFilter(name="target", choices=PUBLIEDITOR_CAR_TARGET_CHOICES)
    seat = django_filters.MultipleChoiceFilter(name="seat", choices=PUBLIEDITOR_CAR_SEAT_CHOICES)
    engine = django_filters.MultipleChoiceFilter(name="engine", choices=PUBLIEDITOR_CAR_ENGINE_CHOICES)
    traction = django_filters.MultipleChoiceFilter(name="traction", choices=PUBLIEDITOR_CAR_TRACTION_CHOICES)
    agear = django_filters.MultipleChoiceFilter(name="agear", choices=PUBLIEDITOR_CAR_AGEAR_CHOICES)
    engine_loc = django_filters.MultipleChoiceFilter(name="engine_loc", choices=PUBLIEDITOR_CAR_ENGINE_LOC_CHOICES)
    layout = django_filters.MultipleChoiceFilter(name="layout", choices=settings.PUBLIEDITOR_CAR_LAYOUT_CHOICES)
    doors = django_filters.MultipleChoiceFilter(name="doors", choices=PUBLIEDITOR_CAR_DOORS_CHOICES)
    fuel = django_filters.MultipleChoiceFilter(name="fuel", choices=PUBLIEDITOR_CAR_FUEL_CHOICES)
    pollution = django_filters.MultipleChoiceFilter(name="pollution", choices=PUBLIEDITOR_CAR_POLLUTION_CHOICES)
    layout = django_filters.MultipleChoiceFilter(name="layout", choices=PUBLIEDITOR_CAR_LAYOUT_CHOICES)
    area_c = django_filters.BooleanFilter(name="area_c")
    driving = django_filters.BooleanFilter(name="driving")
    min_year = django_filters.NumberFilter(name="year", lookup_type='gte')
    max_year = django_filters.NumberFilter(name="year", lookup_type='lte')
    min_luggage = django_filters.NumberFilter(name="luggage", lookup_type='gte')
    max_luggage = django_filters.NumberFilter(name="luggage", lookup_type='lte')
    min_tank = django_filters.NumberFilter(name="tank", lookup_type='gte')
    max_tank = django_filters.NumberFilter(name="tank", lookup_type='lte')
    min_weight = django_filters.NumberFilter(name="weight", lookup_type='gte')
    max_weight = django_filters.NumberFilter(name="weight", lookup_type='lte')
    min_length = django_filters.NumberFilter(name="length", lookup_type='gte')
    max_length = django_filters.NumberFilter(name="length", lookup_type='lte')
    min_width = django_filters.NumberFilter(name="width", lookup_type='gte')
    max_width = django_filters.NumberFilter(name="width", lookup_type='lte')
    min_height = django_filters.NumberFilter(name="height", lookup_type='gte')
    max_height = django_filters.NumberFilter(name="height", lookup_type='lte')
    min_displ = django_filters.NumberFilter(name="displ", lookup_type='gte')
    max_displ = django_filters.NumberFilter(name="displ", lookup_type='lte')
    min_power_cv = django_filters.NumberFilter(name="power_cv", lookup_type='gte')
    max_power_cv = django_filters.NumberFilter(name="power_cv", lookup_type='lte')
    min_torque_max = django_filters.NumberFilter(name="torque_max", lookup_type='gte')
    max_torque_max = django_filters.NumberFilter(name="torque_max", lookup_type='lte')
    min_speed = django_filters.NumberFilter(name="speed", lookup_type='gte')
    max_speed = django_filters.NumberFilter(name="speed", lookup_type='lte')
    min_acc = django_filters.NumberFilter(name="acc", lookup_type='gte')
    max_acc = django_filters.NumberFilter(name="acc", lookup_type='lte')
    min_co2 = django_filters.NumberFilter(name="co2", lookup_type='gte')
    max_co2 = django_filters.NumberFilter(name="co2", lookup_type='lte')
    min_cons_mix = django_filters.NumberFilter(name="cons_mix", lookup_type='gte')
    max_cons_mix = django_filters.NumberFilter(name="cons_mix", lookup_type='lte')
    min_kml_mix = django_filters.NumberFilter(name="kml_mix", lookup_type='gte')
    max_kml_mix = django_filters.NumberFilter(name="kml_mix", lookup_type='lte')
    min_eco = django_filters.NumberFilter(name="eco", lookup_type='gte')
    max_eco = django_filters.NumberFilter(name="eco", lookup_type='lte')
    min_average_electric_consumption = django_filters.NumberFilter(name="average_electric_consumption", lookup_type='gte')
    max_average_electric_consumption = django_filters.NumberFilter(name="average_electric_consumption", lookup_type='lte')
    min_total_power_cv = django_filters.NumberFilter(name="total_power_cv", lookup_type='gte')
    max_total_power_cv = django_filters.NumberFilter(name="total_power_cv", lookup_type='lte')
    min_total_torque_Nm = django_filters.NumberFilter(name="total_torque_Nm", lookup_type='gte')
    max_total_torque_Nm = django_filters.NumberFilter(name="total_torque_Nm", lookup_type='lte')
    min_electric_autonomy_in_km = django_filters.NumberFilter(name="electric_autonomy_in_km", lookup_type='gte')
    max_electric_autonomy_in_km = django_filters.NumberFilter(name="electric_autonomy_in_km", lookup_type='lte')
    min_electric_engine_power_kw = django_filters.NumberFilter(name="electric_engine_power_kw", lookup_type='gte')
    max_electric_engine_power_kw = django_filters.NumberFilter(name="electric_engine_power_kw", lookup_type='lte')

    class Meta:
        model = Car
        fields = settings.PUBLIEDITOR_API_COMMON_FIELDS + ['model']


class BrandFilter(django_filters.FilterSet):
    min_price = django_filters.NumberFilter(name="cars__price", lookup_type="gte")
    max_price = django_filters.NumberFilter(name="cars__price", lookup_type="lte")
    shape = django_filters.MultipleChoiceFilter(name="cars__shape", choices=PUBLIEDITOR_CAR_SHAPE_CHOICES)
    target = django_filters.MultipleChoiceFilter(name="cars__target", choices=PUBLIEDITOR_CAR_TARGET_CHOICES)
    seat = django_filters.MultipleChoiceFilter(name="cars__seat", choices=PUBLIEDITOR_CAR_SEAT_CHOICES)
    engine = django_filters.MultipleChoiceFilter(name="cars__engine", choices=PUBLIEDITOR_CAR_ENGINE_CHOICES)
    traction = django_filters.MultipleChoiceFilter(name="cars__traction", choices=PUBLIEDITOR_CAR_TRACTION_CHOICES)
    agear = django_filters.MultipleChoiceFilter(name="cars__agear", choices=PUBLIEDITOR_CAR_AGEAR_CHOICES)
    engine_loc = django_filters.MultipleChoiceFilter(name="cars__engine_loc",
                                                     choices=PUBLIEDITOR_CAR_ENGINE_LOC_CHOICES)
    layout = django_filters.MultipleChoiceFilter(name="cars__layout", choices=settings.PUBLIEDITOR_CAR_LAYOUT_CHOICES)
    doors = django_filters.MultipleChoiceFilter(name="cars__doors", choices=PUBLIEDITOR_CAR_DOORS_CHOICES)
    fuel = django_filters.MultipleChoiceFilter(name="cars__fuel", choices=settings.PUBLIEDITOR_CAR_FUEL_CHOICES)
    pollution = django_filters.MultipleChoiceFilter(name="cars__pollution", choices=PUBLIEDITOR_CAR_POLLUTION_CHOICES)
    layout = django_filters.MultipleChoiceFilter(name="cars__layout", choices=PUBLIEDITOR_CAR_LAYOUT_CHOICES)
    area_c = django_filters.BooleanFilter(name="cars__area_c")
    driving = django_filters.BooleanFilter(name="cars__driving")
    min_year = django_filters.NumberFilter(name="cars__year", lookup_type='gte')
    max_year = django_filters.NumberFilter(name="cars__year", lookup_type='lte')
    min_luggage = django_filters.NumberFilter(name="cars__luggage", lookup_type='gte')
    max_luggage = django_filters.NumberFilter(name="cars__luggage", lookup_type='lte')
    min_tank = django_filters.NumberFilter(name="cars__tank", lookup_type='gte')
    max_tank = django_filters.NumberFilter(name="cars__tank", lookup_type='lte')
    min_weight = django_filters.NumberFilter(name="cars__weight", lookup_type='gte')
    max_weight = django_filters.NumberFilter(name="cars__weight", lookup_type='lte')
    min_length = django_filters.NumberFilter(name="cars__length", lookup_type='gte')
    max_length = django_filters.NumberFilter(name="cars__length", lookup_type='lte')
    min_width = django_filters.NumberFilter(name="cars__width", lookup_type='gte')
    max_width = django_filters.NumberFilter(name="cars__width", lookup_type='lte')
    min_height = django_filters.NumberFilter(name="cars__height", lookup_type='gte')
    max_height = django_filters.NumberFilter(name="cars__height", lookup_type='lte')
    min_displ = django_filters.NumberFilter(name="cars__displ", lookup_type='gte')
    max_displ = django_filters.NumberFilter(name="cars__displ", lookup_type='lte')
    min_power_cv = django_filters.NumberFilter(name="cars__power_cv", lookup_type='gte')
    max_power_cv = django_filters.NumberFilter(name="cars__power_cv", lookup_type='lte')
    min_torque_max = django_filters.NumberFilter(name="cars__torque_max", lookup_type='gte')
    max_torque_max = django_filters.NumberFilter(name="cars__torque_max", lookup_type='lte')
    min_speed = django_filters.NumberFilter(name="cars__speed", lookup_type='gte')
    max_speed = django_filters.NumberFilter(name="cars__speed", lookup_type='lte')
    min_acc = django_filters.NumberFilter(name="cars__acc", lookup_type='gte')
    max_acc = django_filters.NumberFilter(name="cars__acc", lookup_type='lte')
    min_co2 = django_filters.NumberFilter(name="cars__co2", lookup_type='gte')
    max_co2 = django_filters.NumberFilter(name="cars__co2", lookup_type='lte')
    min_cons_mix = django_filters.NumberFilter(name="cars__cons_mix", lookup_type='gte')
    max_cons_mix = django_filters.NumberFilter(name="cars__cons_mix", lookup_type='lte')
    min_kml_mix = django_filters.NumberFilter(name="cars__kml_mix", lookup_type='gte')
    max_kml_mix = django_filters.NumberFilter(name="cars__kml_mix", lookup_type='lte')
    min_eco = django_filters.NumberFilter(name="cars__eco", lookup_type='gte')
    max_eco = django_filters.NumberFilter(name="cars__eco", lookup_type='lte')

    class Meta:
        model = Brand
        fields = settings.PUBLIEDITOR_API_COMMON_FIELDS


class CarModelFilter(django_filters.FilterSet):
    min_price = django_filters.NumberFilter(name="cars__price", lookup_type="gte")
    max_price = django_filters.NumberFilter(name="cars__price", lookup_type="lte")
    shape = django_filters.MultipleChoiceFilter(name="cars__shape", choices=PUBLIEDITOR_CAR_SHAPE_CHOICES)
    target = django_filters.MultipleChoiceFilter(name="cars__target", choices=PUBLIEDITOR_CAR_TARGET_CHOICES)
    seat = django_filters.MultipleChoiceFilter(name="cars__seat", choices=PUBLIEDITOR_CAR_SEAT_CHOICES)
    engine = django_filters.MultipleChoiceFilter(name="cars__engine", choices=PUBLIEDITOR_CAR_ENGINE_CHOICES)
    traction = django_filters.MultipleChoiceFilter(name="cars__traction", choices=PUBLIEDITOR_CAR_TRACTION_CHOICES)
    agear = django_filters.MultipleChoiceFilter(name="cars__agear", choices=PUBLIEDITOR_CAR_AGEAR_CHOICES)
    engine_loc = django_filters.MultipleChoiceFilter(name="cars__engine_loc",
                                                     choices=PUBLIEDITOR_CAR_ENGINE_LOC_CHOICES)
    layout = django_filters.MultipleChoiceFilter(name="cars__layout", choices=settings.PUBLIEDITOR_CAR_LAYOUT_CHOICES)
    doors = django_filters.MultipleChoiceFilter(name="cars__doors", choices=PUBLIEDITOR_CAR_DOORS_CHOICES)
    fuel = django_filters.MultipleChoiceFilter(name="cars__fuel", choices=PUBLIEDITOR_CAR_FUEL_CHOICES)
    pollution = django_filters.MultipleChoiceFilter(name="cars__pollution", choices=PUBLIEDITOR_CAR_POLLUTION_CHOICES)
    layout = django_filters.MultipleChoiceFilter(name="cars__layout", choices=PUBLIEDITOR_CAR_LAYOUT_CHOICES)
    area_c = django_filters.BooleanFilter(name="cars__area_c")
    driving = django_filters.BooleanFilter(name="cars__driving")
    min_year = django_filters.NumberFilter(name="cars__year", lookup_type='gte')
    max_year = django_filters.NumberFilter(name="cars__year", lookup_type='lte')
    min_luggage = django_filters.NumberFilter(name="cars__luggage", lookup_type='gte')
    max_luggage = django_filters.NumberFilter(name="cars__luggage", lookup_type='lte')
    min_tank = django_filters.NumberFilter(name="cars__tank", lookup_type='gte')
    max_tank = django_filters.NumberFilter(name="cars__tank", lookup_type='lte')
    min_weight = django_filters.NumberFilter(name="cars__weight", lookup_type='gte')
    max_weight = django_filters.NumberFilter(name="cars__weight", lookup_type='lte')
    min_length = django_filters.NumberFilter(name="cars__length", lookup_type='gte')
    max_length = django_filters.NumberFilter(name="cars__length", lookup_type='lte')
    min_width = django_filters.NumberFilter(name="cars__width", lookup_type='gte')
    max_width = django_filters.NumberFilter(name="cars__width", lookup_type='lte')
    min_height = django_filters.NumberFilter(name="cars__height", lookup_type='gte')
    max_height = django_filters.NumberFilter(name="cars__height", lookup_type='lte')
    min_displ = django_filters.NumberFilter(name="cars__displ", lookup_type='gte')
    max_displ = django_filters.NumberFilter(name="cars__displ", lookup_type='lte')
    min_power_cv = django_filters.NumberFilter(name="cars__power_cv", lookup_type='gte')
    max_power_cv = django_filters.NumberFilter(name="cars__power_cv", lookup_type='lte')
    min_torque_max = django_filters.NumberFilter(name="cars__torque_max", lookup_type='gte')
    max_torque_max = django_filters.NumberFilter(name="cars__torque_max", lookup_type='lte')
    min_speed = django_filters.NumberFilter(name="cars__speed", lookup_type='gte')
    max_speed = django_filters.NumberFilter(name="cars__speed", lookup_type='lte')
    min_acc = django_filters.NumberFilter(name="cars__acc", lookup_type='gte')
    max_acc = django_filters.NumberFilter(name="cars__acc", lookup_type='lte')
    min_co2 = django_filters.NumberFilter(name="cars__co2", lookup_type='gte')
    max_co2 = django_filters.NumberFilter(name="cars__co2", lookup_type='lte')
    min_cons_mix = django_filters.NumberFilter(name="cars__cons_mix", lookup_type='gte')
    max_cons_mix = django_filters.NumberFilter(name="cars__cons_mix", lookup_type='lte')
    min_kml_mix = django_filters.NumberFilter(name="cars__kml_mix", lookup_type='gte')
    max_kml_mix = django_filters.NumberFilter(name="cars__kml_mix", lookup_type='lte')
    min_eco = django_filters.NumberFilter(name="cars__eco", lookup_type='gte')
    max_eco = django_filters.NumberFilter(name="cars__eco", lookup_type='lte')

    class Meta:
        model = CarModel
        fields = settings.PUBLIEDITOR_API_COMMON_FIELDS