from rest_framework import serializers

from publieditor.content.models import (
    Brand,
    CarModel,
    Car,
    PhotoCarModelGallery
)


# Model serializers

class BrandSerializer(serializers.ModelSerializer):

    logo = serializers.CharField(source='get_logo_url_big', read_only=True)
    url = serializers.CharField(source='get_absolute_url', read_only=True)
    numberOfVersions = serializers.IntegerField(source='car_count', read_only=True)
    numberOfModels = serializers.IntegerField(source='model_count', read_only=True)

    class Meta:
        model = Brand
        # fields = ('id', 'logo', 'url', 'numberOfVersions', 'numberOfModels', 'name')


class CarModelSerializer(serializers.ModelSerializer):

    image = serializers.CharField(source='get_photo_url_medium', read_only=True)
    url = serializers.CharField(source='get_absolute_url', read_only=True)
    numberOfVersions = serializers.IntegerField(source='cars_count', read_only=True)

    class Meta:
        model = CarModel
        # fields = ('id', 'image', 'url', 'numberOfVersions', 'name')


class CarSerializer(serializers.ModelSerializer):

    url = serializers.CharField(source='get_absolute_url', read_only=True)
    # isAreaC = serializers.BooleanField(source='is_area_c', read_only=True)
    # isDriving = serializers.BooleanField(source='is_driving', read_only=True)  # Is ok for driving beginners

    class Meta:
        model = Car
        # fields = ('id', 'url',)


class BrandModelListSerializer(serializers.ModelSerializer):
    brand_id = serializers.IntegerField(source='brand_id', read_only=True)
    brand_name = serializers.CharField(source='brand.name', read_only=True)
    brand_page_url = serializers.URLField(source='brand.get_absolute_url', read_only=True)
    brand_logo_url = serializers.URLField(source='brand.get_logo_url_big', read_only=True)
    model_id = serializers.IntegerField(source='id', read_only=True)
    model_name = serializers.CharField(source='name', read_only=True)
    model_photo_url = serializers.URLField(source='get_photo_url_medium', read_only=True)
    model_page_url = serializers.URLField(source='get_absolute_url', read_only=True)

    class Meta:
        model = CarModel
        fields = (
            'brand_id', 'brand_name', 'brand_page_url', 'brand_logo_url',
            'model_id', 'model_name', 'model_page_url', 'model_photo_url'
        )


class PhotoSerializer(serializers.ModelSerializer):
    index = serializers.SerializerMethodField('get_index')
    url = serializers.CharField(source='get_absolute_url', read_only=True)
    caption = serializers.SerializerMethodField('get_caption')
    width = serializers.SerializerMethodField('get_width')
    height = serializers.SerializerMethodField('get_height')

    def get_index(self, obj):
        return obj.id

    def get_caption(self, obj):
        return ""

    def get_width(self, obj):
        return obj.get_size()['width']

    def get_height(self, obj):
        return obj.get_size()['height']

    class Meta:
        model = PhotoCarModelGallery
        fields = ('index', 'url', 'caption', 'width', 'height')


class ModelPhotoSerializer(serializers.ModelSerializer):

    title = serializers.CharField(source='name')
    images = PhotoSerializer(source='photos', many=True)

    class Meta:
        model = CarModel
        fields = ('title', 'images')


# Results serializers

class BrandResultsSerializer(serializers.Serializer):

    layout = serializers.CharField()
    data = BrandSerializer()
    count = serializers.IntegerField()


class CarModelResultsSerializer(serializers.Serializer):

    layout = serializers.CharField()
    data = CarModelSerializer()
    count = serializers.IntegerField()


class CarResultsSerializer(serializers.Serializer):

    layout = serializers.CharField()
    data = CarSerializer()
    count = serializers.IntegerField()